;*****************************************************************************
;				NICST TOOL
;
;  EDU RC-5 test for TEB bands. Calculates the Blackbody Calibration Source (BCS)
;  radiance for band M13 only from the tests using the TMC.
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) rad_TMC[collect,band,HAM,detector,subsample]
;  3) rad_p1[band,collect,2] -- 0 for BCS and 1 for SVS.
;  4) rad_p2[band,HAM,collect,5] -- 0 for HAM, 1 for RTA, 2 for CAV, 3 for SH, 
;     and 4 for OBC.
;  5) rvs[band,subsample,detector,HAM,2] -- 0 for TMC and 1 for SVS.
;  6) ncollects
;  
;  v1.0 - 04/20/2008 - Jeff McIntire
;*****************************************************************************

;*****************************************************************************
;  Main program.
;*****************************************************************************

PRO RC5_Gain_rad_1, infile

;*****************************************************************************
;  Read in input file and sav TEB file.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
Free_Lun,lun

n_collect,inpath,ncollects,type,0

restore,'/home/jmcintire/VIIRS_Data/general.TEB.sav'

;*****************************************************************************
;  Some definitions.
;*****************************************************************************

restore,param_in

max_scan=100					; # of scans.

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

outfile=outpath+'/'+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
print,outfile

rad_p1=dblarr(ncollects,nbands_TEB,maxdet,2,2)
rad_p2=dblarr(ncollects,nbands_TEB,nham,maxdet,5,2)

rvs=dblarr(nbands_TEB,maxsubsets,maxdet,nham,2)

rad_TMC=dblarr(ncollects,nbands_TEB,nham,maxdet,2)

;*****************************************************************************
;  Calculate angles.
;*****************************************************************************

win_num=2

; Starting position of the collect window in samples.
MSample_offset=1064*win_num

Msample_max=1439+MSample_offset
Msample_min=1340+MSample_offset

; Center sample numbers.
Msample_center=0.5*(Msample_max+Msample_min)

; Bore-sight scan angles.
Theta_M=((Msample_center-33.5)*0.017785-56.063)*(!pi/180)

Theta_SVS=-65.68*(!pi/180)

Theta_ref=46*(!pi/180)

; HAM angle of incidence.
Phi_perpendicular=28.6*(!pi/180)
Phi=dblarr(nbands_TEB,2)

Phi[3,0]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_M-Theta_ref)))*(180/!pi)
Phi[3,1]=acos(cos(Phi_perpendicular)*cos(0.5*(Theta_SVS-Theta_ref)))*(180/!pi)

;*****************************************************************************
;  Read in scan averaged temperatures for the HAM, RTA, CAV, OBC BB,
;  SH, CPFA, OMM, LWASP, MWASP, BCS, and SVS.
;*****************************************************************************

restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'
print,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'

;*****************************************************************************
;  Calculate the radiances, both integrating the RSR and L over wavelength 
;  and also the Planck radiance.
;*****************************************************************************

; Loop over BCS, rad_p1(i=0), SVS, rad_p1(i=1), HAM, rad_p2(i=0), RTA, rad_p2(i=1),
; CAV, rad_p2(i=2), SH, rad_p2(i=3), OBC, rad_p2(i=4). rad_p3 is the same as rad_p2.

restore,rsrpath

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR col=0, ncollects-1 DO BEGIN
		rad_p1[col,b,0:detectors_TEB[b]-1,0,0]=get_avg_2(rsrf,T_tmc[col],b,detectors_TEB[b])
		rad_p1[col,b,0:detectors_TEB[b]-1,1,0]=get_avg_2(rsrf,T_svs[col],b,detectors_TEB[b])
		FOR ih=0, nham-1 DO BEGIN
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,0,0]=get_avg_2(rsrf,T_ham[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,1,0]=get_avg_2(rsrf,T_rta[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,2,0]=get_avg_2(rsrf,T_cav[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,3,0]=get_avg_2(rsrf,T_sh[ih,col],b,detectors_TEB[b])
			rad_p2[col,b,ih,0:detectors_TEB[b]-1,4,0]=get_avg_2(rsrf,T_obc[ih,col],b,detectors_TEB[b])
		ENDFOR
	ENDFOR
ENDFOR

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR col=0, ncollects-1 DO BEGIN
		FOR id=0, detectors_TEB[b]-1 DO BEGIN
			rad_p1[col,b,id,0,1]=Planck(T_tmc[col],lambda_TEB[b])
			rad_p1[col,b,id,1,1]=Planck(T_svs[col],lambda_TEB[b])
			FOR ih=0, nham-1 DO BEGIN
				rad_p2[col,b,ih,id,0,1]=Planck(T_ham[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,1,1]=Planck(T_rta[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,2,1]=Planck(T_cav[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,3,1]=Planck(T_sh[ih,col],lambda_TEB[b])
				rad_p2[col,b,ih,id,4,1]=Planck(T_obc[ih,col],lambda_TEB[b])
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the RVS.
;*****************************************************************************

; Restore RVS data called COEFF=dblarr(# bands,subsample,# detectors,HAM side,three fit parameters (ao,a1,a2)).
restore,rvs_in
print,rvs_in

FOR b=0, nbands_TEB-1 DO BEGIN

	; 0 for TMC and 1 for SVS.
	FOR j=0, 1 DO BEGIN
		IF band_type_TEB[b] EQ 1 THEN BEGIN
			rvs[b,*,*,*,j]=(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*Phi[b,j]+COEF[b+6,*,*,*,2]*Phi[b,j]^2) $
				/(COEF[b+6,*,*,*,0]+COEF[b+6,*,*,*,1]*38.53+COEF[b+6,*,*,*,2]*38.53^2)
		ENDIF
		IF band_type_TEB[b] EQ 0 THEN BEGIN
			rvs[b,*,*,*,j]=(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*Phi[b,j]+COEF[b-2,*,*,*,2]*Phi[b,j]^2) $
				/(COEF[b-2,*,*,*,0]+COEF[b-2,*,*,*,1]*38.53+COEF[b-2,*,*,*,2]*38.53^2)
		ENDIF
	ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate the TMC radiance.
;*****************************************************************************

FOR b=0, nbands_TEB-1 DO BEGIN
	FOR rsr=0, 1 DO BEGIN
		FOR ih=0, nham-1 DO BEGIN
			FOR col=0, ncollects-1 DO BEGIN
				rad_TMC[col,b,ih,0:detectors_TEB[b]-1,rsr] $
					=p[b]*(rvs[b,0,0:detectors_TEB[b]-1,ih,0]*rad_p1[col,b,0:detectors_TEB[b]-1,0,rsr] $
					-rvs[b,0,0:detectors_TEB[b]-1,ih,1]*rad_p1[col,b,0:detectors_TEB[b]-1,1,rsr]) $
					+(rvs[b,0,0:detectors_TEB[b]-1,ih,1]-rvs[b,0,0:detectors_TEB[b]-1,ih,0]) $
					*(rad_p2[col,b,ih,0:detectors_TEB[b]-1,0,rsr] $
					-(1-p[b])*rad_p2[col,b,ih,0:detectors_TEB[b]-1,1,rsr])
			ENDFOR
		ENDFOR
	ENDFOR
ENDFOR

save,/compress,ncollects,rad_TMC,rad_p1,rad_p2,rvs,p,filename=outfile

END

